<?php

namespace Drupal\follow\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\UserData;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Block with the follow links.
 *
 * @Block(
 *   id = "follow_block",
 *   admin_label = @Translation("Follow links"),
 *   category = @Translation("User")
 * )
 */
class FollowUserBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Returns the current_route_match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Returns the user.data service.
   *
   * @var \Drupal\user\UserData
   */
  protected $userData;

  /**
   * Constructs a FollowUserBlock block.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   Provides an interface for classes representing the result of routing.
   * @param \Drupal\user\UserData $user_data
   *   Defines the user data service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, RouteMatchInterface $route_match, UserData $user_data) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->routeMatch = $route_match;
    $this->userData = $user_data;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('user.data')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];

    /** @var \Drupal\user\UserInterface $user */
    $user = $this->routeMatch->getParameter('user');

    if (!$user || !($user instanceof UserInterface) || !empty($user->in_preview)) {
      return $build;
    }

    $links = follow_get_follow_links();
    $data = (array) $this->userData->get('follow', $user->id(), 'links');

    $items = [];
    foreach ($data as $key => $link) {
      if (!$link || !isset($links[$key])) {
        continue;
      }

      $items[] = [
        'label' => $links[$key]['label'],
        'src' => '/' . $links[$key]['path'] . "/small/icon-{$key}.png",
        'url' => $link,
        'key' => $key,
      ];
    }

    if ($items) {
      $build['follow_links'] = [
        '#theme' => 'follow_links',
        '#links' => $items,
        '#cache' => [
          'tags' => $user->getCacheTags(),
          'contexts' => ['url.path'],
        ],
      ];
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function blockAccess(AccountInterface $account) {
    $access = $account->hasPermission('view follow links');

    return AccessResult::allowedIf($access);
  }

}
